# Python script
# by Jakub Imriska, ITP, ETH Zurich, 2013-2014
# for course on Mechanics of Continua by Vadim Geshkenbein (FS 2014)
# Homework 10.4
# Task: find the stationary 2-dimensional flow of viscous incompressible fluid around a rectangular barrier

# MAIN MODULE WITH SELFCONSISTENCY LOOP TO SOLVE THE COUPLED EQUATIONS FOR STREAMFUNCTION AND VORTICITY

import core_routines
import plotting


# technical constants influencing the selfconsistency loop
relaxation_rate = 0.25    # has to be positive; lower values increase stability, but slow down the convergence
max_iterations  = 800     # upper limit on iterations of the selfconsistency
aimed_precision = 1e-12   # accuracy aimed in the selfconsistently searched solution

# initialize the stream function and vorticity with initial guess
stream_function,vorticity = core_routines.initialize()

# iterative solution of the coupled equations
diff = 2*aimed_precision
it = 0
print "Iteration : difference in stream function, difference in vorticity"
while diff>aimed_precision and it<max_iterations:
  stream_function,diff1 = core_routines.solve_stream_func(stream_function,vorticity,relaxation_rate)
  vorticity,diff2 = core_routines.solve_vorticity(stream_function,vorticity,relaxation_rate)
  diff = max(diff1, diff2)
  print it,':',diff1, diff2
  it+=1
  if diff>1e+8:
    print "Differences blow up, selfconsistent treatment failed. Try to lower the relaxation_rate."
    break

# plotting the results: stream function and vorticity
plotting.show(stream_function,vorticity)
